/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * @(#) $Id: XMAKeyAdapter.java 2605 2008-06-20 13:21:29Z gub $
 *
 *
 *
 *
 *
 */
package at.spardat.xma.event.swt;

import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;

import at.spardat.xma.page.IDialogPage;
import at.spardat.xma.page.PageClient;

/**
 * Event save adapter for key listeners on SWT-widgets not supported by GuiDesigner,
 * like composites, shell, etc. It wraps event-disabling code of XMA around your
 * event handling code. (<a href="package-summary.html#package_description">see package description</>)
 */
public class XMAKeyAdapter implements KeyListener {
    PageClient page;

    /**
     * Constructor for this event adapter.
     * @param page The page containing the widget on which you want to listen for events.
     */
    public XMAKeyAdapter(PageClient page) {
        this.page=page;
    }

    /**
     * Returns the page passed to the contructor
     */
    public PageClient getPage() {
        return page;
    }

    /**
     * This method is called every time a key is pressed on the keyboard.
     * Overload this method and place your envent handling code here.
     * @param event The event send by SWT.
     */
    public void keyPressedImpl(KeyEvent event) {}

    /**
     * The method called by SWT whenever the corresponding event happends.
     * To not overload this method, overload {@link #keyPressedImpl(KeyEvent)}
     * instead.
     * @param event The event send by SWT.
     */
    public final void keyPressed(KeyEvent event) {
        if(!page.isEventsEnabled()) return;
        IDialogPage dialog = page.getDialogPage();
        try {
            dialog.setEventsEnabled(false);
            keyPressedImpl(event);
        } catch (Exception exc) {
            page.showException(exc);
        } finally {
            dialog.setEventsEnabled(true);
        }
    }

    /**
     * This method is called every time a key is released on the keyboard.
     * Overload this method and place your envent handling code here.
     * @param event The event send by SWT.
     */
    public void keyReleasedImpl(KeyEvent event) {}

    /**
     * The method called by SWT whenever the corresponding event happends.
     * To not overload this method, overload {@link #keyReleasedImpl(KeyEvent)}
     * instead.
     * @param event The event send by SWT.
     */
    public final void keyReleased(KeyEvent event) {
        if(!page.isEventsEnabled()) return;
        IDialogPage dialog = page.getDialogPage();
        try {
            dialog.setEventsEnabled(false);
            keyReleasedImpl(event);
        } catch (Exception exc) {
            page.showException(exc);
        } finally {
            dialog.setEventsEnabled(true);
        }
    }

}
