package at.spardat.xma.page.effects;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.List;

import at.spardat.xma.component.ComponentClient;

public class DefaultControlColoring implements IControlColoring {
    
    final static private String EDITABLE_BACKGROUND_COLOR_KEY = "editableBackgroundColor";    
    
    IPageEffects pageEffects;
    
    public DefaultControlColoring(IPageEffects pageEffects) {
        this.pageEffects = pageEffects;
    }
    
    /**
     * This method is called every time the error state is changed for a widget
     */
    public void setBackgroundForErrorState(Control control, boolean editable, boolean inError) {
        if (control instanceof List || control instanceof Combo) {
            setBackgroundForErrorStateList(control,editable,inError);
        } else { 
            Color standardBackgroundColor = getEditableBackground(control);             
            if (inError) {   
                Color errorBackgroundColor = getInvalidControlBackground(control,standardBackgroundColor,editable);
                control.setBackground(errorBackgroundColor);                
            }
            else {
                Color backgroundUneditable = getValidControlBackground(control,standardBackgroundColor,editable);
                control.setBackground(backgroundUneditable);  
            }
        }
    }

    protected void setBackgroundForErrorStateList(Control control, boolean editable, boolean inError ) {
        Color standardBackgroundColor = getEditableBackground(control);                         
        if (inError) {  
            Color errorBackgroundColor = getInvalidControlBackground(control,standardBackgroundColor,false);
            control.setBackground(errorBackgroundColor);        
        } else {
            Color backgroundUneditable = getValidControlBackground(control,standardBackgroundColor,editable);
            control.setBackground(backgroundUneditable);                                               
        }          
    }
    
    public void setEditableBackground(Control control, Color color) {
        Color previousEditableBackground = getEditableBackground(control);
        control.setData(EDITABLE_BACKGROUND_COLOR_KEY,color);        
        if (previousEditableBackground == control.getBackground()) { // .equals(...) would be wrong here
                                                                     // because instances have to be the same
            control.setBackground(color);
        }     
    }    
    
    protected Color getEditableBackground(Control control) {
        Color editableBackground = (Color)control.getData(EDITABLE_BACKGROUND_COLOR_KEY);        
        if (editableBackground == null) {
            editableBackground = control.getBackground();
            control.setData(EDITABLE_BACKGROUND_COLOR_KEY,editableBackground);               
        }  
        return editableBackground;
    }    
    
    protected Color getInvalidControlBackground(Control control, Color editableBackground, boolean editable) {
        if (editable) {
            return pageEffects.getPage().getComponent().getColor(ComponentClient.errorBackground);
        } else {
            return pageEffects.getPage().getComponent().getColor(ComponentClient.errorBackgroundUneditable);
        }                
    }
    
    protected Color getValidControlBackground(Control control, Color editableBackground, boolean editable) {
        if (editable) {
            return editableBackground;
        } else {
            return control.getParent().getBackground();
        }                
    }    

}
