/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * Created on 29.10.2003
 *
 * 
 * 
 */
package at.spardat.xma.security;

import javax.security.auth.spi.LoginModule;

/**
 * This is the extended Interface for LoginModules which supports any
 * changes in the XMAContext after the first login. Such changes can be
 * user switches, mandant switches or similar.
 * 
 * @author s2877
 */
public interface LoginModuleWithContextChange extends LoginModule {
    /**
     * Get if the context change is allowed.
     * @return true if any change in the XMAContext is allowed.
     */
    boolean istContextChangeAllowed();
    
    /**
     * Decide if any change in the XMAContext after the inital login is allowed.
     * @param allowed true to allow, false to disallow
     */
    void setContextChangeAllowed(boolean allowed);

    /**
     * Adds a ContextChangeListener to this LoginModule. All listeners are notified
     * of all changed in the XMAContext after the inital login.
     * @param listener the new listener to register
     */
    void addContextChangeListener(ContextChangeListener listener);

    /**
     * Remove a ContextChangeListener form the listners of the LoginModule. This
     * listener will not be notified of changes in the XMAContext any more.
     * @param listener the listener to remove
     */
    void removeContextChangeListener(ContextChangeListener listener);
}
